/* $Id: CrowdAuthTagLib.groovy 41036 2008-05-09 15:27:22Z caulfield14 $ */

import org.codehaus.grails.plugins.crowd.util.CrowdAuthUtils


/**
 * Tag library to provide convenience tags dealing with authentication and
 * authorization levels for the logged-in user.
 */
class CrowdAuthTagLib {
    static namespace= 'crowdAuth'

    /**
     * Renders the tag body when the user is a member of all supplied Groups and Roles.
     * attribute group a comma-separated list of all Crowd Groups required for authorization
     * attribute role a comma-separated list of all Crowd Roles required for authorization
     */
    def ifAllGranted = { attrs, body ->
        def allGroupsFound= false, allRolesFound= false


        if (attrs['group']) {
            if (CrowdAuthUtils.getAuthenticatedPrincipalGroups(request)?.containsAll(attrs['group']?.split(',')?.toList())) {
                allGroupsFound= true
            }
        }
        else {
            allGroupsFound= true
        }

        if (attrs['role']) {
            if (CrowdAuthUtils.getAuthenticatedPrincipalRoles(request)?.containsAll(attrs['roles']?.split(',')?.toList())) {
                allRolesFound= true
            }
        }
        else {
            allRolesFound= true
        }

        if (allGroupsFound && allRolesFound) {
            out << body()
        }
    }

    /**
     * Renders the tag body when the user is a member of any supplied Groups and Roles.
     * attribute group a comma-separated list of all Crowd Groups required for authorization
     * attribute role a comma-separated list of all Crowd Roles required for authorization
     */
    def ifAnyGranted = { attrs, body ->
        def groupFound= false, roleFound= false

        if (attrs['group']) {
            for (group in attrs['group']?.split(',')?.toList()) {
                if (CrowdAuthUtils.getAuthenticatedPrincipalGroups(request)?.contains(group)) {
                    groupFound= true
                }
            }
        }
        else {
            groupFound= true
        }

        if (attrs['role']) {
            for (role in attrs['role']?.split(',')?.toList()) {
                if (CrowdAuthUtils.getAuthenticatedPrincipalRoles(request)?.contains(role)) {
                    roleFound= true
                }
            }
        }
        else {
            roleFound= true
        }

        if (groupFound || roleFound) {
            out << body()
        }
    }

    /**
     * Renders the tag body when the user is not a member of all of the supplied Groups and Roles.
     * attribute group a comma-separated list of all Crowd Groups required for authorization
     * attribute role a comma-separated list of all Crowd Roles required for authorization
     */
    def ifAllNotGranted = { attrs, body ->
        def groupFound= false, roleFound= false

        if (attrs['group']) {
            for (group in attrs['group']?.split(',')?.toList()) {
                if (CrowdAuthUtils.getAuthenticatedPrincipalGroups(request)?.contains(group)) {
                    groupFound= true
                }
            }
        }

        if (attrs['role']) {
            for (role in attrs['role']?.split(',')?.toList()) {
                if (CrowdAuthUtils.getAuthenticatedPrincipalRoles(request)?.contains(role)) {
                    roleFound= true
                }
            }
        }

        if (!(groupFound || roleFound)) {
            out << body()
        }
    }

    /**
     * Renders the tag body when the user is not a member of one of the supplied Groups and Roles.
     * attribute group a comma-separated list of all Crowd Groups required for authorization
     * attribute role a comma-separated list of all Crowd Roles required for authorization
     */
    def ifAnyNotGranted = { attrs, body ->
        def allGroupsFound= false, allRolesFound= false


        if (attrs['group']) {
            if (CrowdAuthUtils.getAuthenticatedPrincipalGroups(request)?.containsAll(attrs['group']?.split(',')?.toList())) {
                allGroupsFound= true
            }
        }

        if (attrs['role']) {
            if (CrowdAuthUtils.getAuthenticatedPrincipalRoles(request)?.containsAll(attrs['roles']?.split(',')?.toList())) {
                allRolesFound= true
            }
        }

        if (!(allGroupsFound && allRolesFound)) {
            out << body()
        }
    }

    /**
     * Renders the tag body when the user is authenticated in the application.
     */
    def isAuthenticated = { attrs, body ->
        log.debug("CrowdAuthTagLib.isAuthenticated()")
        if (CrowdAuthUtils.isAuthenticated(request)) {
            out << body()
        }
    }

    /**
     * Renders the tag body when the user is not authenticated in the application.
     */
    def isNotAuthenticated = { attrs, body ->
        log.debug("CrowdAuthTagLib.isNotAuthenticated()")
        if (!CrowdAuthUtils.isAuthenticated(request)) {
            out << body()
        }
    }

    /**
     * Renders information about the current user.
     * attribute property the property to display
     */
    def authenticatedUserInfo = { attrs ->
        switch (attrs['property']) {
        case 'username':
            out << CrowdAuthUtils.getAuthenticatedPrincipal(request)?.name
            break
        case 'displayName':
            out << CrowdAuthUtils.getAuthenticatedPrincipal(request)?.
                    attributes?.
                    toList()?.
                    find() { attribute ->
                        attribute.name == 'displayName'
                    }?.
                    values
            break
        case 'name':
            def firstName= CrowdAuthUtils.getAuthenticatedPrincipal(request)?.attributes?.toList()?.find() { attribute -> attribute.name == 'givenName' }?.values?.join()
            def lastName= CrowdAuthUtils.getAuthenticatedPrincipal(request)?.attributes?.toList()?.find() { attribute -> attribute.name == 'sn' }?.values?.join()
            out << "${firstName} ${lastName}"
            break
        case 'firstName':
            out << CrowdAuthUtils.getAuthenticatedPrincipal(request)?.
                    attributes?.
                    toList()?.
                    find() { attribute ->
                        attribute.name == 'givenName'
                    }?.
                    values
            break
        case 'lastName':
            out << CrowdAuthUtils.getAuthenticatedPrincipal(request)?.
                    attributes?.
                    toList()?.
                    find() { attribute ->
                        attribute.name == 'sn'
                    }?.
                    values
            break
        case 'email':
            out << CrowdAuthUtils.getAuthenticatedPrincipal(request)?.
                    attributes?.
                    toList()?.
                    find() { attribute ->
                        attribute.name == 'mail'
                    }?.
                    values
            break
        case 'telephoneNumber':
            out << CrowdAuthUtils.getAuthenticatedPrincipal(request)?.
                    attributes?.
                    toList()?.
                    find() { attribute ->
                        attribute.name == 'telephoneNumber'
                    }?.
                    values
            break
        case 'faxNumber':
            out << CrowdAuthUtils.getAuthenticatedPrincipal(request)?.
                    attributes?.
                    toList()?.
                    find() { attribute ->
                        attribute.name == 'facsimileTelephoneNumber'
                    }?.
                    values
            break
        }
    }
}
